class Endpoints:
    host = 'http://cobalt.qas.im/api/1.0'

    @staticmethod
    def run(api, get, endpoint=None, params=None, filter_keys=None):
        endpoint = Endpoints._parse_endpoint(endpoint)

        url, params = Endpoints._parse_url(api, endpoint, params), \
            Endpoints._parse_params(endpoint, params, filter_keys)

        return get(url=url, params=params)

    @staticmethod
    def _process_filter(queries, filter_keys=None):
        if (type(queries) == str):
            return queries

        a = []
        for filter in queries:
            o = []
            for k, v in filter:
                if filter_keys and k.lower() not in filter_keys:
                    continue
                o.append('%s:%s' % (k, v))
            a.append(' OR '.join(o))
        return ' AND '.join(a)

    @staticmethod
    def _parse_endpoint(endpoint='list'):
        if not endpoint or endpoint in ('list', 'show'):
            return ''

        return endpoint if endpoint in ('search', 'filter') else ''

    @staticmethod
    def _parse_url(api, endpoint, params):
        url = '%s/%s/%s' % (Endpoints.host, api, endpoint)

        if not params:
            return url

        keys = map(lambda p: p.lower(), params.keys())

        if endpoint == '' and any(k in ('date', 'id') for k in keys):
            url += params['id'] if 'id' in params else params['date']

        return url

    @staticmethod
    def _parse_params(endpoint, params, filter_keys=None):

        if not params:
            return None

        keys = map(lambda p: p.lower(), params.keys())

        if endpoint == '' and any(k in ('date', 'id') for k in keys):
            return None

        if endpoint in ('filter', 'search') and 'q' not in keys:
            raise ValueError('Expected a query parameter with this endpoint.')

        parsed_params = {}

        for param, value in params.items():
            param = param.lower()

            if param not in ('sort', 'limit', 'skip', 'q'):
                continue

            if endpoint == 'filter' and param == 'q':
                value = Endpoints._process_filter(queries=value,
                                                  filter_keys=filter_keys)

            parsed_params[param] = value

        return parsed_params



from .endpoints import Endpoints
from .helpers import get, get_filter_keys


class Cobalt:
    def __init__(self, api_key=None):
        self.host = 'http://cobalt.qas.im/api/1.0'

        self.headers = {
            'Referer': 'https://pypi.python.org/pypi/cobaltuoft',
            'Authorization': api_key
        }

        if not api_key or not self._is_valid_key():
            raise ValueError('Expected valid API key.')

        self.filter_keys = get_filter_keys()

    def _get(self, url, params=None, headers=None):
        headers = headers or self.headers
        return get(url=url, params=params, headers=headers)

    def _is_valid_key(self):
        r = self._get(self.host)
        return r.reason == 'Not Found' and r.status_code == 404

    def _run(self, api, endpoint=None, params=None):
        res = Endpoints.run(api=api,
                            endpoint=endpoint,
                            params=params,
                            filter_keys=self.filter_keys[api],
                            get=self._get)
        return res.json()

    def athletics(self, endpoint=None, params=None):
        return self._run(api='athletics', endpoint=endpoint, params=params)

    def buildings(self, endpoint=None, params=None):
        return self._run(api='buildings', endpoint=endpoint, params=params)

    def courses(self, endpoint=None, params=None):
        return self._run(api='courses', endpoint=endpoint, params=params)

    def food(self, endpoint=None, params=None):
        return self._run(api='food', endpoint=endpoint, params=params)

    def textbooks(self, endpoint=None, params=None):
        return self._run(api='textbooks', endpoint=endpoint, params=params)



import requests
import json

from bs4 import BeautifulSoup

BASE_URL = 'https://cobalt.qas.im/documentation/%s/filter'


def scrape(api):
    host = BASE_URL % api
    resp = requests.get(host)
    soup = BeautifulSoup(resp.text, 'html.parser')

    filters = []

    if not soup.find('table'):
        return filters

    for tr in soup.find('table').find_all('tr'):
        if tr.find('th') or not tr.find('td'):
            continue

        filters.append(tr.find('td').text)

    return filters


def main(active_apis):
    filters = {}
    for api in active_apis:
        filters[api] = scrape(api)
    return filters

if __name__ == '__main__':
    main()



import requests
import json

from bs4 import BeautifulSoup

BASE_URL = 'https://cobalt.qas.im'


def main():
    resp = requests.get(BASE_URL)
    soup = BeautifulSoup(resp.text, 'html.parser')

    if not soup.find(id='apis'):
        return []

    return [a.text.lower() for a in soup.find(id='apis').find_all('a')]

if __name__ == '__main__':
    main()



import json
import requests

from collections import OrderedDict
from datetime import datetime
from os.path import exists, isfile

from .scrapers import *


def get(url, params=None, headers=None):
    with requests.Session() as s:
        return s.get(url=url, params=params, headers=headers, timeout=10)


def deep_convert_dict(obj):
    converted = obj

    if isinstance(obj, OrderedDict):
        converted = dict(obj)
    elif isinstance(obj, list):
        converted = [deep_convert_dict(x) for x in obj]

    try:
        for k, v in converted.items():
            converted[k] = deep_convert_dict(v)
    except AttributeError:
        pass

    return converted


def verify_dir_exists(directory):
    if not exists(directory):
        os.makedirs(directory)

    return exists(directory)


def get_active_apis():
    if not verify_dir_exists('./data'):
        return []

    if isfile('./data/active_apis.json'):
        with open('./data/active_apis.json', 'r') as f:
            doc = json.loads(f.read())

        write_date = datetime.strptime(doc['meta']['date'],
                                       '%Y-%m-%d %H:%M:%S')

        # Rescrape if previous data is more than 5 days old
        if (datetime.now() - write_date).seconds <= 432000:
            return doc['apis']

    doc = {
        'apis': scrape_apis(),
        'meta': {
            'date': datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        }
    }

    with open('./data/active_apis.json', 'w') as f:
        f.write(json.dumps(doc, indent=2))

    return doc['apis']


def get_filter_keys():
    if not verify_dir_exists('./data'):
        return []

    if isfile('./data/filter_keys.json'):
        with open('./data/filter_keys.json', 'r') as f:
            doc = json.loads(f.read())

        write_date = datetime.strptime(doc['meta']['date'],
                                       '%Y-%m-%d %H:%M:%S')

        # Rescrape if previous data is more than 5 days old
        if (datetime.now() - write_date).seconds <= 432000:
            return doc['filters']

    doc = {
        'filters': scrape_filter_keys(get_active_apis()),
        'meta': {
            'date': datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        }
    }

    with open('./data/filter_keys.json', 'w') as f:
        f.write(json.dumps(doc, indent=2))

    return doc['filters']



from setuptools import setup

setup(
    name='cobaltuoft',
    version='0.0.5',
    description='A wrapper for the Cobalt API.',
    long_description='A Python library for interfacing with Cobalt (http://cobalt.qas.im), a University of Toronto Open Data API.',
    url='https://github.com/kshvmdn/cobalt-uoft-python',
    author='Kashav Madan',
    author_email='kshvmdn@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords=['cobalt', 'uoft', 'university of toronto', 'api wrapper', 'open data'],
    packages=[
        'cobaltuoft',
        'cobaltuoft.endpoints',
        'cobaltuoft.helpers',
        'cobaltuoft.helpers.scrapers'
    ],
    package_data={
        '': ['LICENSE']
    },
    package_dir={'cobaltuoft': 'cobaltuoft'},
    install_requires=['requests', 'bs4'],
    include_package_data=True
)



import json

from collections import OrderedDict
from os.path import splitext

from .helpers import get, deep_convert_dict


class Datasets:
    GH_API = 'https://api.github.com'
    API_URL = '%s/repos/cobalt-uoft/datasets' % GH_API

    @staticmethod
    def _get(url, params=None):
        return get(url=url, params=params, headers={
            'Referer': Datasets.GH_API
        })

    @staticmethod
    def _get_tags():
        resp = Datasets._get(url='%s/tags' % Datasets.API_URL)
        return list(map(lambda tag: tag['name'], resp.json()))

    @staticmethod
    def _get_available_datasets(tag):
        resp = Datasets._get(url='%s/contents' % Datasets.API_URL,
                             params={'ref': tag})

        datasets = {}

        for file in resp.json():
            if '.json' in file['name']:
                datasets[splitext(file['name'])[0]] = {
                    'name': file['name'],
                    'url': file['download_url']
                }

        return datasets

    @staticmethod
    def _parse_cobalt_json(json_string):
        docs = []

        for doc in json_string.strip().split('\n'):
            try:
                docs.append(json.loads(doc, object_pairs_hook=OrderedDict))
            except e:
                pass

        return deep_convert_dict(docs)

    @staticmethod
    def run(tag='latest', datasets='*'):
        tag = None if tag is None else tag.lower()

        if not tag or tag == 'latest':
            tag = 'master'

        if tag != 'master' and tag not in Datasets._get_tags():
            raise ValueError('Unexpected tag value. Refer to this: ' +
                             'https://api.github.com/repos/cobalt-uoft/datasets/tags ' +
                             'for a list of valid tags.')

        available_datasets = Datasets._get_available_datasets(tag)

        if not datasets:
            raise ValueError('Unexpected datasets value.')
        elif datasets == '*':
            datasets = available_datasets.keys()
        elif type(datasets) != list:
            datasets = [datasets]

        datasets = [ds.lower() for ds in datasets]

        docs = {}

        for name, doc in available_datasets.items():
            if name not in datasets:
                continue

            resp = Datasets._get(url=doc['url'])

            if resp.status_code != 200:
                continue

            docs[name] = Datasets._parse_cobalt_json(resp.text)

        return docs
